/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-07-31 17:19:06.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.service.impl;

import cn.com.qmth.examcloud.app.core.exception.ApiException;
import cn.com.qmth.examcloud.app.core.utils.*;
import cn.com.qmth.examcloud.app.model.*;
import cn.com.qmth.examcloud.app.service.PropertyService;
import cn.com.qmth.examcloud.app.service.RedisService;
import cn.com.qmth.examcloud.app.service.UserAuthService;
import okhttp3.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.HashMap;
import java.util.Map;

import static cn.com.qmth.examcloud.app.model.Constants.*;

/**
 * 认证中心业务服务接口
 *
 * @author: fengdesheng
 * @since: 2018/7/16
 */
@Service
public class UserAuthServiceImpl implements UserAuthService {
    private static Logger log = LoggerFactory.getLogger(UserAuthServiceImpl.class);
    @Autowired
    private PropertyService propertyService;
    @Autowired
    private RedisService redisService;

    @Override
    public Result<User> login(LoginInfo loginInfo) throws Exception {
        Assert.notNull(loginInfo, "LoginInfo must be not null.");
        if (StringUtils.isBlank(loginInfo.getAccountType())) {
            loginInfo.setAccountType(LoginType.COMMON_LOGIN_NAME.name());
        }
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_core/auth/login", propertyService.getUserAuthUrl());
        Map<String, String> params = new HashMap<>();
        params.put("accountValue", loginInfo.getAccount());
        params.put("password", loginInfo.getPassword());
        params.put("accountType", loginInfo.getAccountType());
        params.put("rootOrgId", loginInfo.getRootOrgId() != null ? loginInfo.getRootOrgId().toString() : "");
        params.put("domain", loginInfo.getDomain());
        String json = new JsonMapper().toJson(params);
        RequestBody formBody = FormBody.create(MediaType.parse(Constants.CHARSET_JSON_UTF8), json);
        Request request = new Request.Builder()
                .url(requestUrl)
                .post(formBody)
                .build();
        //执行请求
        Response response = HttpBuilder.client.getInstance().newCall(request).execute();
        String bodyStr = response.body().string();
        if (response.isSuccessful()) {
            //获取用户信息
            User userInfo = new JsonMapper().fromJson(bodyStr, User.class);
            return new Result().success(userInfo);
        } else {
            log.warn("Http response is " + bodyStr);
            ResBody body = new JsonMapper().fromJson(bodyStr, ResBody.class);
            if (body != null && body.getCode() != null) {
                return new Result().error(body.getDesc());
            }
            return new Result().error(bodyStr);
        }
    }

    @Override
    public Result logout(String key, String token) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_core/auth/logout", propertyService.getUserAuthUrl());
        RequestBody formBody = new FormBody.Builder()
                .add(PARAM_KEY, key)
                .add(PARAM_TOKEN, token)
                .build();
        return HttpUtils.doPost(requestUrl, formBody, key, token);
    }

    @Override
    public Result getUserInfo(String key, String token) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_core/auth/getLoginUser", propertyService.getUserAuthUrl());
        RequestBody formBody = new FormBody.Builder()
                .add(PARAM_KEY, key)
                .add(PARAM_TOKEN, token)
                .build();
        //todo 待补充'头像、手机号、年级'字段
        return HttpUtils.doPost(requestUrl, formBody, key, token);
    }

    @Override
    public Result updatePassword(String key, String token, Long userId, String password) throws Exception {
        Assert.notNull(userId, "UserId must be not null.");
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_core/user/password", propertyService.getUserAuthUrl());
        RequestBody formBody = new FormBody.Builder()
                .add("userId", userId.toString())
                .add("password", password)
                .build();
        return HttpUtils.doPut(requestUrl, formBody, key, token);
    }

    @Override
    public Result userBindingPhone(String key, String token, Long userId, String phone, String code) {
        //todo
        return new Result().error();
    }

    @Override
    public void cacheLoginInfo(LoginInfo loginInfo, User userInfo) {
        if (loginInfo == null || userInfo == null) {
            return;
        }
        //封装原始登录信息
        loginInfo.setKey(userInfo.getKey());
        loginInfo.setToken(userInfo.getToken());
        //将原始登录信息存放到Redis中
        String redisKey = StrUtils.md5Key(userInfo.getKey());
        this.cacheLoginInfo(loginInfo, redisKey);
        log.debug("redisKey:" + redisKey + " loginToken:" + loginInfo.getToken());
        //替换login token为缓存的redisKey，并作为接口的结果返回
        userInfo.setToken(redisKey);
    }

    /**
     * 缓存用户登录信息
     */
    @Override
    public void cacheLoginInfo(LoginInfo loginInfo, String key) {
        if (StringUtils.isEmpty(key)) {
            throw new ApiException("Key must be not empty.");
        }
        redisService.set(APP_SESSION_USER_KEY_PREFIX + key, new JsonMapper().toJson(loginInfo), APP_SESSION_EXPIRE_TIME);
    }

    /**
     * 获取缓存中的用户登录信息
     */
    @Override
    public LoginInfo getLoginInfo(String key) {
        if (StringUtils.isEmpty(key)) {
            throw new ApiException("Key must be not empty.");
        }
        String loginInfo = redisService.get(APP_SESSION_USER_KEY_PREFIX + key);
        if (loginInfo != null) {
            return new JsonMapper().fromJson(loginInfo, LoginInfo.class);
        }
        return null;
    }

    /**
     * 获取平台端的默认过期时间(秒)
     */
    @Override
    public int getSessionTimeout() {
        try {
            String timeout = redisService.get(Constants.PLATFORM_SESSION_TIMEOUT_KEY);
            if (StringUtils.isNotEmpty(timeout)) {
                return Integer.parseInt(timeout);
            }
        } catch (Exception e) {
            //ignore
        }
        return Constants.PLATFORM_SESSION_EXPIRE_TIME;
    }

    /**
     * 初始化内部接口请求鉴权
     */
    @Override
    public void initRequestTrace() {
        String key = "C_" + ThreadUtils.getTraceID();
        Long millis = System.currentTimeMillis();
        redisService.set(key, millis.toString(), 10);
    }

}